#
# Monitoring Suite, Seven Solutions S.L.
#
# This file is part of monitoring-7s.
# You might use, distribute and modify this code under the terms of
# the LICENSE.txt provided within the project monitoring-7s.
#
# If you do not have received a copy of the LICENSE.txt along with
# this file please write to info@sevensols.com and consider that
# this file can not be copied and/or distributed in any forms.
#
# Created by Eva Lopez Ruiz in June 2021.
#

"""Monitoring suite configuration."""


from wrz_mon.utils.filesystem import create_directory
from wrz_mon.utils import yml_dump_file, yml_load_file
from wrz_mon.utils import json_load_file, json_dump_file
from wrz_mon.utils import LOGGER
from datetime import datetime
from re import TEMPLATE, match
import os


"""Configuration file path"""
DEFAULT_CONF_FILE = 'config.yml'
"""Schema file path"""
DEFAULT_SCHEMA_FILE = 'wrz_mon/configuration/config_schema.yml'


"""Directory to save old configuration files"""
DATA_DIR = '.wrzmon'
OLD_CONFIGS_DIR = '.wrzmon/old'
TOKENS_FILE = '.wrzmon/tokens'


"""Containers names"""
GRAFANA_CONTAINER = 'wrz-grafana'
INFLUXDB_CONTAINER = 'wrz-influxdb'
TELEGRAF_CONTAINER = 'wrz-telegraf'


"""Configuration fields"""
MAIN_FIELDS = ['global', 'devices', 'auto-discovery']
GLOBAL_F, DEVICES_F, AUTODISCOVERY_F = MAIN_FIELDS

"""Global fields"""
GLOBAL_FIELDS = ['organization', 'timezone', 'notifications-type',
                 'email', 'ports', 'bucket']
ORGANIZATION_F, TIMEZONE_F, NOTTYPE_F, EMAIL_F, PORTS_F, BUCKET_F = GLOBAL_FIELDS

OPT_PARAMETERS = ['username', 'password', 'influxdb_token', 'grafana_token']
USERNAME_F, PASSWORD_F, INFLUXDB_TOKEN_F, GRAFANA_TOKEN_F = OPT_PARAMETERS

TOKEN_F = 'token'

SECOND_USER_F = 'second_user'
SECOND_PASSWORD_F = 'second_password'

"""Global-Ports fields"""
GRAFANA_F = 'grafana'
INFLUXDB_F = 'influxdb'

"""Auto-discovery fields"""
NETWORK_F = 'network'

"""Devices fields"""
DEVICES_FIELDS = ['agents', 'device-type', 'version', 'sec_name',
                  'auth_password', 'priv_password', 'template']
AGENTS_F, DEVICE_TYPE_F, VERSION_F, SEC_NAME_F, AUTH_PASSWORD_F, PRIV_PASSWORD_F, TEMPLATE_F = DEVICES_FIELDS
AGENTS_FIELDS = ['ip', 'port', 'protocol']
IP_F, PORT_F, PROTOCOL_F = AGENTS_FIELDS

class Configuration:
    """The Configuration object represent the Monitoring suite configuration.

    Note:
        It is convenient to create a unique object of this class that is
        shared with all provisioning functions and classes. That's why this
        class is a singleton.

    Parameters:
        __instance (Configuration): To know if exist an instance of this class
        globa_conf (dict): Global configuration section. It contains bucket,
        Grafana and InfluxDB ports organization, timezone, email and
        notifications type.
        auto-discovery (dict): Auto-discovery configuration
        devices (dict): Devices to monitor
    """

    __instance = None

    def __new__(cls, *args, **kwargs):
        """Return the Configuration instance. If it doesn't exist, create it.
        """
        if not Configuration.__instance:
            Configuration.__instance = object.__new__(cls)
        return Configuration.__instance

    def __init__(self):
        """Initializa attributes to None value."""
        self._devices = None
        self._global_conf = None
        self._autodiscovery = None
        self._auth = dict()
        self._old_config = None

    @property
    def username(self):
        try:
            return self._auth[USERNAME_F]
        except Exception:
            return None

    @username.setter
    def username(self, value):
        self._auth[USERNAME_F] = value

    @property
    def password(self):
        try:
            return self._auth[PASSWORD_F]
        except Exception:
            return None

    @password.setter
    def password(self, value):
        if len(value) > 7:
            self._auth[PASSWORD_F] = value
        else:
            LOGGER.error('Unable to set password. It must be least at 8 characters.')

    @property
    def second_user(self):
        try:
            return self._auth[SECOND_USER_F]
        except Exception:
            return None

    @second_user.setter
    def second_user(self, value):
        self._auth[SECOND_USER_F] = value

    @property
    def second_password(self):
        try:
            return self._auth[SECOND_PASSWORD_F]
        except Exception:
            return None

    @second_password.setter
    def second_password(self, value):
        if len(value) > 7:
            self._auth[SECOND_PASSWORD_F] = value
        else:
            LOGGER.error('Unable to set password. It must be least at 8 characters.')

    @property
    def organization(self):
        try:
            return self._global_conf[ORGANIZATION_F]
        except Exception:
            return None

    @property
    def timezone(self):
        try:
            return self._global_conf[TIMEZONE_F]
        except Exception:
            return None

    @property
    def notifications_type(self):
        try:
            return self._global_conf[NOTTYPE_F]
        except Exception:
            return None

    @property
    def email(self):
        try:
            return self._global_conf[EMAIL_F]
        except Exception:
            return None

    @property
    def grafana_port(self):
        try:
            return self._global_conf[PORTS_F][GRAFANA_F]
        except Exception:
            return None

    @property
    def grafana_token(self):
        try:
            return self._auth[GRAFANA_TOKEN_F]
        except Exception:
            return None

    @grafana_token.setter
    def grafana_token(self, value):
        self._auth[GRAFANA_TOKEN_F] = value

    @property
    def influxdb_port(self):
        return self._global_conf[PORTS_F][INFLUXDB_F]

    @property
    def influxdb_token(self):
        try:
            return self._auth[INFLUXDB_TOKEN_F]
        except Exception:
            return None

    @influxdb_token.setter
    def influxdb_token(self, value):
        self._auth[INFLUXDB_TOKEN_F] = value

    @property
    def bucket(self):
        try:
            return self._global_conf[BUCKET_F]
        except Exception:
            return None

    @property
    def network(self):
        try:
            return self._autodiscovery[NETWORK_F]
        except Exception:
            return None

    @property
    def devices(self):
        return self._devices

    def add_agent(self, group, hostname, new_agent):
        """Add new agent in a group"""
        if group not in self._devices.keys():
            LOGGER.warning('Unable to add agent {0}. Group {1} not found'.format(hostname, group))
            return

        if AGENTS_F not in self._devices[group].keys():
            self._devices[group][AGENTS_F] = dict()

        self._devices[group][AGENTS_F][hostname] = new_agent

    def load(self, conf_f=DEFAULT_CONF_FILE, schema_f=DEFAULT_SCHEMA_FILE):
        """Read and validate a YML configuration file.

        Parameters:
            conf_f (str): Configuration file to read
            schema_f (str): File to validate the configuration

        Returns:
            err (str): None value on success, otherwise exception caught
        """
        config, err = yml_load_file(conf_f, schema_f)
        if config is None:
            LOGGER.error('Unable to load configuration. Exception: %s' % err)
            return False

        self._devices = config[DEVICES_F]
        self._global_conf = config[GLOBAL_F]
        self._autodiscovery = config[AUTODISCOVERY_F]
        self._global_conf[BUCKET_F] = "7S"

        last_copy = self._search_previous_config()
        if last_copy:
            last_copy = os.path.join(OLD_CONFIGS_DIR, last_copy)
            self._old_config, err = yml_load_file(last_copy, None)

        return True

    def write(self, filename=DEFAULT_CONF_FILE):
        """Write the configuration in filename"""
        config = dict()
        config[DEVICES_F] = self._devices
        config[GLOBAL_F] = self._global_conf
        config[AUTODISCOVERY_F] = self._autodiscovery
        retval = yml_dump_file(filename, config)
        return retval

    def save(self):
        """Save the configuration in .old directory."""
        now = datetime.now()
        fname = "config_{0}.conf".format(now.strftime("%Y-%m-%d_%H:%M:%S"))

        # It may not exist
        create_directory(DATA_DIR)
        create_directory(OLD_CONFIGS_DIR)

        config_files = os.listdir(OLD_CONFIGS_DIR)
        for f in config_files:

            df = match(r'config_(\d+)-(\d+)-(\d+)_(\d+):(\d+):(\d+).conf', f)
            if df is None:
                continue

            datef = datetime(int(df[1]), int(df[2]), int(df[3]),
                             int(df[4]), int(df[5]), int(df[6]))
            if datef > now:
                warn_msg = "Exist a copy newer than {}. It won't save" % fname
                LOGGER.warn(warn_msg)
                return

        fname = os.path.join(OLD_CONFIGS_DIR, fname)
        retval = self.write(fname)
        if retval != 0:
            LOGGER.error('Unable to save a copy of the configuration')
            return

    def devices_equals_old(self):
        """Compare the section devices of current configuration with
        the previous configuration.

        Returns:
            True if global section has changed, otherwise False.
        """
        try:
            return self._old_config[DEVICES_F] == self._devices
        except Exception:
            return False

    def global_equals_old(self):
        """Compare the section global of current configuration with
        the previous configuration.

        Returns:
            True if the configurations are equal, otherwise False.
        """
        try:
            return self._old_config[GLOBAL_F] == self._global_conf
        except Exception:
            return False

    def organization_equals_old(self):
        """Compare the current organization name with the previous
        configuration.

        Returns:
            True if the organization names are equal, otherwise False
        """
        try:
            return self._old_config[GLOBAL_F][ORGANIZATION_F] == self._global_conf[ORGANIZATION_F]
        except Exception:
            return False

    def email_equals_old(self):
        """Compare the current email with the previous
        configuration.

        Returns:
            True if the emails are equal, otherwise False
        """
        try:
            return self._old_config[GLOBAL_F][EMAIL_F] == self._global_conf[EMAIL_F]
        except Exception:
            return False

    def read_token(self, program):
        """Read program token from .token file.

        Params:
            program (str): Program's token

        Returns:
            token (string): token on success, otherwise None value
        """
        tokens, err = json_load_file(TOKENS_FILE)
        if tokens is None:
            err = 'Unable to read {0} token. Exception: {1}'.format(program, err)
            LOGGER.error(err)
            return

        if program not in tokens.keys():
            LOGGER.warn('Token not found')
            return

        if program == GRAFANA_CONTAINER:
            self._auth[GRAFANA_TOKEN_F] = tokens[program]
        elif program == INFLUXDB_CONTAINER:
            self._auth[INFLUXDB_TOKEN_F] = tokens[program]

    def write_token(self, program):
        """Write program token to .token file.

        Params:
            program (str): Program's token
            token (str): Token to write

        Returns:
            retval (int): 0 on success, otherwise -1
        """
        tokens, err = json_load_file(TOKENS_FILE)
        if tokens is None:
            create_directory(DATA_DIR)
            tokens = dict()

        if program == GRAFANA_CONTAINER:
            tokens[program] = self._auth[GRAFANA_TOKEN_F]
        elif program == INFLUXDB_CONTAINER:
            tokens[program] = self._auth[INFLUXDB_TOKEN_F]

        err = json_dump_file(TOKENS_FILE, tokens)
        if err:
            err = 'Failure to write {0} token. Exception: {1}'.format(program, err)
            LOGGER.warn(err)
            return -1

        return 0

    def _search_previous_config(self):
        """Search the last config saved in old folder."""
        try:
            config_files = os.listdir(OLD_CONFIGS_DIR)
        except Exception:
            LOGGER.info('Old configuration files not found')
            return None

        lastc = None
        lastc_date = datetime(1950, 2, 8, 0, 0, 0)

        for f in config_files:

            df = match(r'config_(\d+)-(\d+)-(\d+)_(\d+):(\d+):(\d+).conf', f)
            if df is None:
                continue

            datef = datetime(int(df[1]), int(df[2]), int(df[3]),
                             int(df[4]), int(df[5]), int(df[6]))
            if datef > lastc_date:
                lastc = f
                lastc_date = datef

        return lastc
