#
# Monitoring Suite, Seven Solutions S.L.
#
# This file is part of monitoring-7s.
# You might use, distribute and modify this code under the terms of
# the LICENSE.txt provided within the project monitoring-7s.
#
# If you do not have received a copy of the LICENSE.txt along with
# this file please write to info@sevensols.com and consider that
# this file can not be copied and/or distributed in any forms.
#
# Created by Eva Lopez Ruiz in June 2021.
#

"""Grafana Setup Manager."""


import os
from wrz_mon.utils import LOGGER
from wrz_mon.utils import json_load_file, json_dump_file
from wrz_mon.comunication import GrafanaClient
from wrz_mon.configuration import Configuration, GRAFANA_CONTAINER, DATA_DIR


CONFIG = Configuration()

"""Grafana parameters"""
GRAFANA_DIR = 'grafana'
DASHBOARDS_DIR = 'dashboards'
DATASOURCE_FILE = 'datasources/influxdb.json'


def grafana_set_initial_setup():
    """Configure Grafana service."""
    grafana_url = 'localhost:{}'.format(CONFIG.grafana_port)
    grafana_client = GrafanaClient(grafana_url)

    if grafana_client.check_service() != 0:
        return -1

    LOGGER.info('Grafana service started')

    CONFIG.grafana_token = grafana_client.generate_token()
    if CONFIG.grafana_token is None:
        return -1

    CONFIG.write_token(GRAFANA_CONTAINER)

    if grafana_client.update_organization(CONFIG.organization) != 0:
        return -1

    retval = grafana_client.update_admin_user(CONFIG.username,
                                              CONFIG.password,
                                              CONFIG.email)
    if retval != 0:
        return -1

    if CONFIG.second_user and CONFIG.second_password:
        grafana_client.create_user(CONFIG.second_user, CONFIG.second_password)

    grafana_client.create_folder('Custom')

    if grafana_create_datasource(grafana_client) != 0:
        return -1
    if grafana_import_dashboards(grafana_client) != 0:
        return -1

    return 0


def grafana_update(new_username=None, new_password=None):
    """Update grafana configuration."""
    CONFIG.read_token(GRAFANA_CONTAINER)
    if CONFIG.grafana_token is None:
        return

    grafana_url = 'localhost:{}'.format(CONFIG.grafana_port)
    grafana_client = GrafanaClient(url=grafana_url,
                                   token=CONFIG.grafana_token,
                                   username=CONFIG.username,
                                   password=CONFIG.password)

    if CONFIG.organization_equals_old() is False:
        grafana_client.update_organization(CONFIG.organization)

    if new_username and new_password:
        grafana_client.update_admin_user(new_username,
                                         new_password,
                                         CONFIG.email)
    elif (None in (new_username, new_username) and
          CONFIG.email_equals_old() is False):
        grafana_client.update_admin_user(CONFIG.username,
                                         CONFIG.password,
                                         CONFIG.email)


def grafana_create_datasource(grafana_client=None):
    """
    """
    if grafana_client is None:

        CONFIG.read_token(GRAFANA_CONTAINER)
        if CONFIG.grafana_token is None:
            LOGGER.error('Unable to import dashboard')
            return -1

        grafana_url = 'localhost:{}'.format(CONFIG.grafana_port)
        grafana_client = GrafanaClient(grafana_url, token=CONFIG.grafana_token)

    datasource_id_path = DATA_DIR + '/datasource_id.json'
    datasource_id, err = json_load_file(datasource_id_path)
    if datasource_id is None:
        datasource_id = dict()

    path = os.path.join(GRAFANA_DIR, DATASOURCE_FILE)
    content, err = json_load_file(path)
    if content is None:
        err = 'Unable to load Grafana datasource: {0}. Exception: {1}'.format(path, err)
        LOGGER.error(err)
        return -1

    content['url'] = 'http://influxdb:{}'.format(CONFIG.influxdb_port)
    content['jsonData']['defaultBucket'] = CONFIG.bucket
    content['jsonData']['organization'] = CONFIG.organization

    datasource_name = content['name']
    id = grafana_client.create_datasource(content)
    if id > 0:
        datasource_id[datasource_name] = id

    err = json_dump_file(datasource_id_path, datasource_id)
    if err:
        LOGGER.error('Unable to dump datasource ID. Exception: %s' % err)
        return -1

    return 0


def grafana_update_datasource(grafana_client=None):
    """
    """
    if grafana_client is None:

        CONFIG.read_token(GRAFANA_CONTAINER)
        if CONFIG.grafana_token is None:
            LOGGER.error('Unable to import dashboard')
            return -1

        grafana_url = 'localhost:{}'.format(CONFIG.grafana_port)
        grafana_client = GrafanaClient(grafana_url, token=CONFIG.grafana_token)

    datasource_id_path = DATA_DIR + '/datasource_id.json'
    datasource_id, err = json_load_file(datasource_id_path)

    if datasource_id is None:
        LOGGER.error('Unable to load datasource ID. Exception: %s' % err)
        return -1

    path = os.path.join(GRAFANA_DIR, DATASOURCE_FILE)
    content, err = json_load_file(path)
    if content is None:
        err = 'Unable to load Grafana datasource: {0}. Exception: {1}'.format(path, err)
        LOGGER.error(err)
        return -1

    datasource_name = content['name']
    if datasource_name not in datasource_id.keys():
        LOGGER.error('Dashboards ID not found')
        return -1

    content['id'] = datasource_id[datasource_name]
    content['url'] = 'http://influxdb:{}'.format(CONFIG.influxdb_port)
    content['jsonData']['defaultBucket'] = CONFIG.bucket
    content['jsonData']['organization'] = CONFIG.organization

    id = grafana_client.update_datasource(content)
    if id > 0:
        datasource_id[datasource_name] = id

    err = json_dump_file(datasource_id_path, datasource_id)
    if err:
        LOGGER.error('Unable to dump datasource ID. Exception: %s' % err)
        return -1

    return 0


def grafana_import_dashboards(grafana_client=None, create_dashboards=True):
    """Import dashboards to Docker container.

    Parameters:
        grafana_client (GrafanaClient object): to communicate with Grafana API
        create_dashboards (boolean): determine if dashboard will be created or
        updated
    """
    if grafana_client is None:

        CONFIG.read_token(GRAFANA_CONTAINER)
        if CONFIG.grafana_token is None:
            LOGGER.error('Unable to import dashboard')
            return -1

        grafana_url = 'localhost:{}'.format(CONFIG.grafana_port)
        grafana_client = GrafanaClient(grafana_url, token=CONFIG.grafana_token)

    dashboards_ids_path = DATA_DIR + '/dashboards_ids.json'
    dashboards_ids, err = json_load_file(dashboards_ids_path)

    if dashboards_ids is None and create_dashboards is True:
        dashboards_ids = dict()
    elif dashboards_ids is None and create_dashboards is False:
        LOGGER.error('Unable to load dashboards IDs. Exception: ' + err)
        return -1

    path = os.path.join(GRAFANA_DIR, DASHBOARDS_DIR)
    dashboard_files = os.listdir(path)

    for file in dashboard_files:

        path = os.path.join(GRAFANA_DIR, DASHBOARDS_DIR, file)
        content, err = json_load_file(path)
        if content is None:
            err = 'Unable to load Grafana dashboard: {0}. Exception: {1}'.format(path, err)
            LOGGER.error(err)
            continue

        if create_dashboards is False and file in dashboards_ids.keys():
            content['dashboard']['id'] = dashboards_ids[file]['id']
            content['dashboard']['version'] = dashboards_ids[file]['version']

        response = grafana_client.import_dashboard(content)
        if response:

            if file not in dashboards_ids.keys():
                dashboards_ids[file] = dict()

            dashboards_ids[file]['id'] = response[0]
            dashboards_ids[file]['version'] = response[1]

    err = json_dump_file(dashboards_ids_path, dashboards_ids)
    if err:
        LOGGER.error('Unable to dump dashboards IDs. Exception: %s' % err)
        return -1

    return 0


def grafana_export_dashboards(grafana_client=None):
    """Export dashboards from Docker container.

    Parameters:
        grafana_client (GrafanaClient object): to communicate with Grafana API
    """
    if grafana_client is None:

        CONFIG.read_token(GRAFANA_CONTAINER)
        if CONFIG.grafana_token is None:
            LOGGER.error('Unable to export dashboard')
            return

        grafana_url = 'localhost:{}'.format(CONFIG.grafana_port)
        grafana_client = GrafanaClient(grafana_url, token=CONFIG.grafana_token)

    LOGGER.info('TODO')
