#
# Monitoring Suite, Seven Solutions S.L.
#
# This file is part of monitoring-7s.
# You might use, distribute and modify this code under the terms of
# the LICENSE.txt provided within the project monitoring-7s.
#
# If you do not have received a copy of the LICENSE.txt along with
# this file please write to info@sevensols.com and consider that
# this file can not be copied and/or distributed in any forms.
#
# Created by Eva Lopez Ruiz in June 2021.
#

"""All classes and functions related to Docker containers management."""


import subprocess


class Container:
    """The Container object represent a docker container instance.

    Attributes:
        name (str): The container name
    """

    def __init__(self, cont_name):
        """Add the container name.

        Parameters:
            cont_name (str): Use to set the container name
        """
        self.name = cont_name

    def copy(self, src, dst):
        """Copy src to dst.

        Parameters:
            src (str): source file(s)/folder
            dst (str): target file(s)/folder

        Returns:
            retval (int): 0 on success, otherwise -1
            stderr (str): None on success, otherwise stderr of the executed command
        """
        cp = subprocess.run(['docker', 'cp', src, dst], capture_output=True)
        if cp.returncode != 0:
            return -1, cp.stderr

        return 0, None

    def restart(self):
        """Restart container.

        Returns:
            retval (int): 0 on success, otherwise -1
            stderr (str): None on success, otherwise stderr of the executed command
        """
        cp = subprocess.run(['docker', 'restart', self.name], capture_output=True)
        if cp.returncode != 0:
            return -1, cp.stderr

        return 0, None


class Compose:
    """Class to manage Docker Compose"""

    def __init__(self):
        """Initializee a Compose object
        """
        self.compose = 'docker-compose'

    def up_containers(self):
        """Execute docker-compose up command to run Docker containers."""
        cp = subprocess.run([self.compose, 'up', '-d'], capture_output=True)
        if cp.returncode != 0:
            return -1, cp.stderr
        return 0, None

    def down_containers(self):
        """Execute docker-compose down command to stop and remove containers"""
        cp = subprocess.run([self.compose, 'down'], capture_output=True)
        if cp.returncode != 0:
            return -1, cp.stderr
        return 0, None
